<!--- begin start mode --->
<cfswitch expression="#ThisTag.ExecutionMode#">


	<cfcase value="start">
		<cfset variables.Error = "">
		<cfset variables.lValidOutputValues = "plain,commas,images">
		<!--- must have a closing tag --->
		<cfif NOT ThisTag.HasEndTag>
			<cfset variables.Error = variables.Error & "<li>A closing tag for CF_FIREFOX_COUNTER is needed</li>">
		</cfif>
		<cfif StructKeyExists(attributes, "Output") AND Len(attributes.Output)>
			<cfif NOT ListFind(variables.lValidOutputValues, attributes.Output)>
				<cfset variables.Error = variables.Error & "<li>Illegal use of the 'Output' parameter</li>">
			</cfif>
		</cfif>
		<!--- print out the error messages --->
		<cfif Len(Trim(variables.Error))>
			<strong>The following Errors have been detected in CF_FIREFOX_COUNTER</strong>
			<ol>
				<cfoutput>#variables.Error#</cfoutput>
			</ol>
			<cfexit>
		</cfif>
		<cfparam name="attributes.CacheTime" type="numeric" default="24">
		<cfparam name="attributes.CacheTimeUnit" type="string" default="h">
		<cfparam name="attributes.Directory" type="string" default="/firefox/">
		<cfparam name="attributes.Filename" type="string" default="firefox_download_counter">
		<cfparam name="attributes.Output" type="string" default="commas">
		<cfparam name="attributes.ImagesDirectory" type="string" default="/firefox/images/">
		<cfparam name="attributes.ImagesExtension" type="string" default="gif">
	</cfcase>
	
	
	<cfcase value="end">
	
		<cfparam name="variables.cCustomTagContent" type="string" default="#ThisTag.GeneratedContent#">
		<cfparam name="variables.cFirefoxFullPath" type="string" default="#ExpandPath(attributes.Directory) & attributes.Filename#.xml">
		<cfparam name="variables.cXmlFileContent" type="string" default="">
		<cfparam name="variables.sXmlDocument" type="struct" default="#StructNew()#">
		<cfparam name="variables.qFolderContents" type="query" default="#QueryNew('empty')#">
		
		<!--- variables used by user in tag UI --->
		<cfparam name="count" type="numeric" default="0">
		
		<cfset ThisTag.GeneratedContent = "">
		
		<cfif FileExists(variables.cFirefoxFullPath)>
			<cftry>
				<cfdirectory action="list" directory="#ExpandPath(attributes.Directory)#" name="variables.qFolderContents" filter="#attributes.Filename#.xml">
				<cfif variables.qFolderContents.RecordCount AND DateCompare(variables.qFolderContents.DateLastModified, DateAdd("#attributes.CacheTimeUnit#", - attributes.CacheTime, Now())) EQ -1>
					<cfset variables.cXmlFileContent = GetXmlFile()>
					<cfset variables.cXmlFileContent = RemoveXmlDocTypeLine(variables.cXmlFileContent)>
					<cfset WriteFile(variables.cXmlFileContent)>
				</cfif>
				<cfcatch>
					<cfoutput>
						#cfcatch.message#<br>
						#cfcatch.detail#
					</cfoutput>
				</cfcatch>
			</cftry>
		<cfelse>
			<cfset variables.cXmlFileContent = GetXmlFile()>
			<cfset variables.cXmlFileContent = RemoveXmlDocTypeLine(variables.cXmlFileContent)>
			<cfset WriteFile(variables.cXmlFileContent)>
		</cfif>
		
		<cftry>
			<cffile action="READ" file="#variables.cFirefoxFullPath#" variable="xmlContents">
			<cfset variables.sXmlDocument = XMLParse(xmlContents)>
			<cfset count = variables.sXmlDocument.rss.channel.item.description.XmlText>
			<cfswitch expression="#attributes.Output#">
				<cfcase value="plain">
					<cfset count = Replace(count, ",", "", "ALL")>
				</cfcase>
				<cfcase value="commas">
				</cfcase>
				<cfcase value="images">
					<cfset count = GetNumberInImages(count)>
				</cfcase>
			</cfswitch>
			<cfoutput>
				<cfif Len(variables.cCustomTagContent)>
					#Evaluate(DE(variables.cCustomTagContent))#
				<cfelse>
					#count#
				</cfif>
			</cfoutput>
			<cfcatch>
				<cfoutput>
					#cfcatch.message#<br>
					#cfcatch.detail#
				</cfoutput>
			</cfcatch>
		</cftry>
		
	</cfcase>
	
	
</cfswitch>


<!--- Common functions --->
<cffunction name="GetXmlFile" access="private" output="false" returntype="string">
	<cfset var cFileContent = "">
 
	<cfhttp method="GET" url="http://feeds.spreadfirefox.com/downloads/firefox.xml" resolveurl="No" charset="utf-8"></cfhttp>
	<cfif cfhttp.StatusCode IS "200 OK">
		<cfset cFileContent = cfhttp.FileContent>
	</cfif>
		
	<cfreturn cFileContent>
</cffunction>


<cffunction name="WriteFile" access="private" output="false">
	<cfargument name="aContent" required="true" type="string">
	
	<cfset var cFullFilePath = ExpandPath(attributes.Directory) & attributes.Filename & ".xml">
	
	<cffile action="WRITE" file="#cFullFilePath#" nameconflict="OVERWRITE" output="#arguments.aContent#">
</cffunction>


<cffunction name="RemoveXmlDocTypeLine" access="private" output="false" returntype="string">
	<cfargument name="cXmlContent" type="string" required="yes">
	
	<cfset var lEditedXml = "">
	<cfset var iDocTypeLine = 0>
	
	<cfset iDocTypeLine = ListContainsNoCase(arguments.cXmlContent, "<!DOCTYPE rss SYSTEM ", chr(10) & chr(13))>
	<cfset lEditedXml = ListDeleteAt(arguments.cXmlContent, iDocTypeLine, chr(10) & chr(13))>
	
	<cfreturn lEditedXml>
</cffunction>


<cffunction name="GetNumberInImages" access="private" output="false" returntype="string">
	<cfargument name="cNumber" required="yes" type="string">
	
	<cfset var cNumberInImages = "">
	
	<cfloop from="1" to="#Len(arguments.cNumber)#" index="letter">
		<cfset cNumberInImages = cNumberInImages & "<img src=""" & attributes.ImagesDirectory>
		<cfif Mid(arguments.cNumber, letter, 1) IS ",">
			<cfset cNumberInImages = cNumberInImages & "comma">
		<cfelse>
			<cfset cNumberInImages = cNumberInImages & Mid(arguments.cNumber, letter, 1)>
		</cfif>
		<cfset cNumberInImages = cNumberInImages & "." & attributes.ImagesExtension & """>">
	</cfloop>
	
	<cfreturn cNumberInImages>
</cffunction>